"""
Admin models and interface for GameTeX.
"""

from django.contrib import admin
from gametex.models import GameTeXObject, GameTeXFieldValue, GameTeXUser

class CustomFieldInline(admin.TabularInline):
    """
    Inline editor for GameTeX custom fields.
    """
    model = GameTeXFieldValue

class GTUInline(admin.TabularInline):
    """
    Inline editor for GameTeX users.
    """
    model = GameTeXUser

class GTOAdmin(admin.ModelAdmin):
    """
    GameTeXObject admin.
    """
    fields = ('name', 'macro', 'classes')
    readonly_fields = ('classes',)
    inlines = [CustomFieldInline, GTUInline]

admin.site.register(GameTeXObject, GTOAdmin)



"""
GameTeX models.
"""

__author__ = 'ternus'


from gametex.models import GameTeXClass, GameTeXField, \
			   GameTeXFieldValue, GameTeXUser, GTO
import json

def import_gametex(filename, preserve=False):
    """
    Import GameTeX from a file.  Assumes file exists and is sane JSON.

    If preserve is True, when merging with existing content, prioritize
    the database's value over the JSON file's.
    """
    print "Importing GameTeX from %s." % filename
    if preserve:
        print "Preserving existing database values."
    gametex_file = file(filename, 'r')
    gametex_json = json.load(gametex_file)
    print "Found %d objects." % len(gametex_json)
    for obj in gametex_json:
        print "Loading %s... " % (obj['macro'] if 'macro' in obj else ''),
        gto = GTO.objects.get_or_create(macro = obj['macro'])
        if not gto[1]:
            print "<found!> ",
        gto = gto[0]
        if not (gto.name and preserve):
            gto.name = obj['name']
        gto.save()
        print '[',
        for cls in obj['classes']:
            clx = GameTeXClass.objects.get_or_create(name = cls)[0]
            gto.classes.add(clx)
            print cls,
        print '] { ',
        for field in obj:
            if field == 'macro' or field == 'name' or field == 'classes':
                continue
            fld = GameTeXField.objects.get_or_create(name = field)[0]
            val = GameTeXFieldValue.objects.get_or_create(field = fld,
                                                          object = gto)
            if val[1] or not preserve:
                val[0].value = obj[field]
                print "%s=%s " % (field, obj[field]),
            else:
                print "%s==%s " % (field, val[0].value),
        print "} done."
    print "Done loading GameTeX objects."

def create_users(force=False):
    """
    Automatically create users from GameTeX objects.
    
    If force is True, ignore errors.
    """
    print "Creating users..."
    from django.contrib.auth.models import User
    pcs = GTO.bc('PC')

    # sanity check
    fail = False
    for char in pcs:
        if not char.has_field('username'):
            print "PC %s (%s) doesn't have username set!" % (char.name,
                                                             char.macro)
            fail = True
        if not char.has_field('password'):
            print "PC %s (%s) doesn't have password set!" % (char.name,
                                                             char.macro)
            fail = True
    if fail and not force:
        print "Couldn't import -- fix errors first."
        return False
    for char in pcs:
        if not (char.has_field('username') and char.has_field('password')):
            continue
        usr = User.objects.create_user(username=char.username,
                                     password=char.password)
        GameTeXUser.objects.get_or_create(gto=char, user=usr)
        print "Created %s" % char.username
    print "Done creating users."



"""
GameTeX models.
"""

from django.db import models
from django.contrib.auth.models import User

class GameTeXClass(models.Model):
    """
    A single GameTeX class in the GameTeX type system.  Many-to-many with GTO.
    Non-hierarchical.
    """
    name = models.CharField(max_length=256, unique=True, primary_key=True)

    def __unicode__(self):
        return self.name

class GameTeXField(models.Model):
    """
    A single defined field (e.g. "password" or "Combat Rating").
    """
    name = models.CharField(max_length=256, unique=True, primary_key=True)

    def __unicode__(self):
        return self.name

class GameTeXFieldValue(models.Model):
    """
    Mapping GameTeXObjects to GameTeXFields with specific values.
    It's how freeform values are stored in a relational database.
    """
    field = models.ForeignKey('GameTeXField')
    object = models.ForeignKey('GameTeXObject')
    value = models.CharField(max_length=256, blank=True)

    def __unicode__(self):
        return "%s %s:%s" % (self.object, self.field, self.value)

class GameTeXObject(models.Model):
    """
    The heart of the system.  Each GameTeX entity
    (sheet, character, GM, keycard, whatever)
    generated by the JSON export gets its own GameTeXObject.
    """
    macro = models.CharField(max_length=256, unique=True, primary_key=True)
    name = models.CharField(max_length=256)
    classes = models.ManyToManyField(GameTeXClass)
    custom_fields = models.ManyToManyField(GameTeXField,
                                          through=GameTeXFieldValue)

    def __unicode__(self):
        return "%s [%s]" % (self.macro, self.name)

    def __getattr__(self, item):
        """
        Allows access to custom fields through dot syntax
        e.g. gto.market
        """
        return self.field(item)

    def __setattr__(self, key, value):
        """
        Allows setting custom fields.
        """
        try:
            self.set_field(key, value)
        except AttributeError:
            return super(GameTeXObject, self).__setattr__(key, value)

    def has_field(self, field):
        """
        Shortcut.  Check whether a field exists.
        """
        return GameTeXFieldValue.objects.filter(object=self,
                                                field=field).exists()

    def field(self, item):
        """
        Access a field by name.
        Useful if it contains spaces ("Combat Rating")
        """
        flds = GameTeXField.objects.filter(name=item)
        if flds.exists():
            vals = GameTeXFieldValue.objects.filter(field=flds[0],
                                                    object=self)
            if vals.exists():
                return vals[0].value
        raise AttributeError

    def set_field(self, key, value):
        """
        Set a field by name.
        """
        try:
            fieldval = GameTeXFieldValue.objects.get(field__name=key,
                                                    object=self)
            fieldval.value = value
            fieldval.save()
        except:
            raise AttributeError

    @classmethod
    def bc(cls, clname):
        """
        Get GTOs by GameTeX class (e.g. 'PC' or 'Item')
        """
        return GameTeXObject.objects.filter(classes__name=clname)


class GameTeXUser(models.Model):
    """
    Mapping between Django auth users and GameTeX PCs.
    """
    gto = models.ForeignKey('GameTeXObject')
    user = models.ForeignKey(User)

# shortcut
GTO = GameTeXObject



from distutils.core import setup
from setuptools import find_packages

setup(
    name='gametex-django',
    version='0.3.3',
    packages=find_packages(),
    author='Christian Ternus',
    author_email='ternus@cternus.net',
    url='http://github.com/ternus/gametex-django',
    license='Creative Commons Attribution-Noncommercial-Share Alike license',
    description='Django integration for GameTeX',
    long_description=open('README.md').read(),
    install_requires=['Django'],
)
